#include "snake.h"
#include <limits.h>	/* PATH_MAX */
#include <stdlib.h>	/* malloc */
#include <string.h>	/* strings */

#define HUMAN_CONFIG "etc/human.conf"

/* external data */
extern int typein;
extern char *rootdir;

/* external functions */
extern int register_player (snake *);
extern snake *snake_make (void);

/* local definitions */
typedef struct t_brain {
	int kl; /* left key */
	int ku; /* up key */
	int kr; /* right key */
	int kd; /* down key */
	int ks; /* stop key */
	int kg; /* grow key */
} brain;

/* private functions */
static void play (snake *);
static brain *make_brain ( int kl, int ku, int kr, int kd, int ks, int kg );

/* public functions */
int module_init (void);


int module_init ()
{
	snake *mysnake;
	brain *mybrain;
	FILE *conf;
	char confpath[PATH_MAX];
	char buf[256];
	int ret;
	char ku, kl, kr, kd; /* snake parameters */
	/* configuration file format */
	char format[] = "human %c %c %c %c\n";

	strcpy(confpath, rootdir);
	strcat(confpath, "/");
	strcat(confpath, HUMAN_CONFIG);

	conf = fopen(confpath, "r");
	if ( !conf ) {
		fprintf(stderr, "error opening %s: %s\n",
			confpath, strerror(errno));
		return -1;
	}

	while ( fgets(buf, 255, conf) )
	{
		int playernum;

		/* read configuration file */
		ret = sscanf(buf, format, &kl, &ku, &kr, &kd);

#ifdef DEBUG
		fprintf (stderr, "%s", buf);
		fprintf (stderr, "sscanf returned %i\n", ret);
		fprintf (stderr, "%c, %c, %c, %c\n", kl, ku, kr, kd);
#endif

		/* invalid record format */
		if ( ret < 4 ) continue;
		
		/* make a brain */
		mybrain = make_brain (kl, ku, kr, kd, 0, 0);
		if ( !mybrain ) { /* out of virtual memory */
			return 1;
		}

		mysnake = snake_make();
		mysnake->brain = mybrain;
		mysnake->play = &play;
		playernum = register_player(mysnake);
		mysnake->look = BE_SNAKE('0'+(playernum%10));
	}

	return 0;
}

static void play (snake *s)
{
	brain *mybrain = (brain *) s->brain;

	if ( typein == mybrain->kl )
	{
		if ( IS_VDIR(s->dir) || s->length == 1 )
			s->dir = LEFT;
	}

	else if ( typein == mybrain->ku )
	{
		if ( IS_HDIR(s->dir) || s->length == 1 )
			s->dir = UP;
	}

	else if ( typein == mybrain->kr )
	{
		if ( IS_VDIR(s->dir) || s->length == 1 )
			s->dir = RIGHT;
	}

	else if ( typein == mybrain->kd )
	{
		if ( IS_HDIR(s->dir) || s->length == 1 )
			s->dir = DOWN;
	}
}


/* allocate a brain structure */
static brain *make_brain ( int kl, int ku, int kr, int kd, int ks, int kg )
{
	brain *mybrain;

	mybrain = (brain *)malloc(sizeof(brain));
	if ( !mybrain ) {
		fprintf(stderr, "out of virtual memory\n");
		return NULL;
	}

	mybrain->kl = kl; /* left key */
	mybrain->ku = ku; /* up key */
	mybrain->kr = kr; /* right key */
	mybrain->kd = kd; /* down key */
	mybrain->ks = ks; /* stop key */
	mybrain->kg = kg; /* grow key */

	return mybrain;
}
