#include "include/snake.h"
#include <stdlib.h>		/* malloc and friends */

/* extern data */
extern snake *zoo[];

/* external functions */
extern void move_cell (cell *, int, int);
extern void snake_kill (snake *);
extern void init_fruit (void);

/* private functions */
static void move_head (snake *);
static void eat_tail (snake *);
static void move_single_snake (snake *);

/* public functions */
void move_snakes (void);

/* move each snake */
void move_snakes ()
{
	int i;

	for (i=0; zoo[i]; i++) {
		if (zoo[i]->died) continue;
		move_single_snake(zoo[i]);
	}
}

/* move head, eat tail */
void move_single_snake ( snake *s )
{
	/* move_head if not stopped */
	if ( s->dir != STOP ) move_head(s);

	/* finish here if snake died */
	if ( s->died ) return;

	/* eat the tail if not growing */
	if ( ! s->grow ) eat_tail(s);
	else s->grow--;
}


/* move the snake's head */
static void move_head ( snake *s )
{
	chain *newhead;
	chtype ch;

	/* allocate memory for new chain */
	newhead = (chain *) malloc(sizeof(chain));

	/* initialize newhead coordinates to old head's */
	newhead->cell.x = s->head->cell.x;
	newhead->cell.y = s->head->cell.y;

	/* move head one step in the desired direction */
	move_cell(&(newhead->cell), s->dir, 1);

	/* get chtype belove the new head */
	ch=GETCHTYPE(newhead->cell.y, newhead->cell.x);

	/* check if snake have hitten something */
	if ( IS_SNAKE(ch) ) {
		snake_kill(s);
		free(newhead);
		return;
	}

	/* check if snake have eaten something */
	if ( IS_FRUIT(ch) ) {
		/* eat a fruit */
		s->grow  += ( (ch&A_CHARTEXT)-48 );
		s->score += (ch&A_CHARTEXT)-48;
		init_fruit();
	}
	else if ( IS_EXSNAKE(ch) ) {
		/* eat the snake's rest */
		s->score ++;
	}

	/* print the new head */
	/* do it (for now) to avoid transparency of 1-lenght snakes */
	PUTCHTYPE(newhead->cell.y, newhead->cell.x, s->look);

	/* set the new snake's head */
	s->head->prev = newhead;
	newhead->next = s->head;
	newhead->prev = NULL;
	
	s->head = newhead;

	/* increment the new snake's length */
	s->length++;
}
	

/* eat snake's tail */
static void eat_tail ( snake *s )
{
 	/* chain pointer */
	chain *ptr;

	/* delete the old tail before its coordinates change */
	/* this shouldn't be done by this module */
	PUTCHTYPE( s->tail->cell.y, s->tail->cell.x, ' ');

	/* find the chain before the tail */
	ptr = s->tail->prev;
	/*for ( ptr=s->head; ptr->next != s->tail; ptr=ptr->next );*/

	/* make it the new tail pointer */
	s->tail = ptr; 

	/* free memory of the old tail */
	free(s->tail->next); 

	/* set the new tail pointer to NULL */
	s->tail->next = NULL;

	/* decrement snake's lenght */
	s->length--;

	/* if snake disappeared, finish it */
	if( s->length == 0 ) snake_kill(s);
}

